from typing import Optional
import requests
from requests.auth import AuthBase


class Client:
    base = 'https://farm.openzim.org/api'
    access_token: Optional[str] = None

    def password_auth(self, username: str, password: str):
        response = requests.post(f'{self.base}/auth/oauth2', json={
            'grant_type': 'password',
            'username': username,
            'password': password
        })

        response_json = response.json()
        if response.status_code == 200:
            self.access_token = response_json.get('access_token')
        else:
            raise Exception()

    def get(self, url: str) -> requests.Response:
        return requests.get(f'{self.base}{url}', auth=BearerTokenAuth(self.access_token))


class BearerTokenAuth(AuthBase):
    def __init__(self, access_token: str):
        self.access_token = access_token

    def __call__(self, request):
        request.headers['Authorization'] = f'Bearer {self.access_token}'
        return request



from typing import Union
from enum import Enum
from datetime import datetime

from .base import Base


class Status(Enum):
    Online = 'online'
    Offline = 'offline'


class Worker(Base):
    def __init__(self, id: str, hostname: str, status: Union[str, Status], heartbeats: [], **kwargs):
        self.id = id
        self.hostname = hostname
        self.status = Status(status)
        self.heartbeats = heartbeats

    def __repr__(self):
        return f'Worker {self.hostname} {self.status.value}'

    @property
    def last_heartbeat(self):
        return self.heartbeats[-1] if self.heartbeats else None

    @classmethod
    def list(cls) -> ['Worker']:
        response = cls.client.get('/workers/')
        if response.status_code == 200:
            workers = []
            for json in response.json():
                json['id'] = json.pop('_id')
                json['heartbeats'] = [datetime.fromisoformat(json.pop('heartbeat').replace('Z', '+00:00'))]
                worker = cls(**json)
                workers.append(worker)
            return workers
        else:
            raise Exception



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zimfarm",
    version="0.0.3",
    author="Chris Li",
    author_email="chris@kiwix.com",
    description="Zimfarm Client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/openzim/zimfarm-client",
    install_requires=["requests>=2.21.0"],
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['zimfarm=zimfarm.commands.zimfarm:main'],
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
    ],
)
